
#include <vector>
#include <string>

#include "TLegend.h"
#include "TPad.h"
#include "TStyle.h"
#include "TFile.h"


class Legend { 
  
public:

  Legend(double x0=0, double x1=0, double y0=0, double y1=0) : tleg(0) { 
    m_x[0] = x0;
    m_x[1] = x1;
    m_y[0] = y0;
    m_y[1] = y1;
  } 

  ~Legend() { 
    //   std::cout << "Legend::~Legend() size " << m_obj.size() << std::endl; 
    if ( tleg ) delete tleg;
  }

  
  TLegend* operator->() { return tleg; } 

  void AddEntry(TObject* t, const std::string& label, const std::string& type="p") { 
    //   std::cout << "Legend::AddEntry() size " << m_obj.size() << std::endl; 
    m_labels.push_back(label);
    m_type.push_back(type);
    m_obj.push_back(t);
  }

  void Header(const std::string& s) { m_header = s; }


  void Draw(double x=0, double y=0, double dy=0.07) { 

    //    std::cout << "Legend: Draw() " << size() << std::endl;
   
    if ( y==0 ) y = m_y[1];
    if ( x==0 ) x = m_x[0];
 
    if ( tleg!=0 ) { 
      delete tleg;
      //      tleg->Draw();
      //      return;
    }

    tleg = new TLegend(x, y-m_obj.size()*dy, x+(m_x[1]-m_x[0]), y);

    tleg->SetFillStyle(4000);
    // tleg->SetFillStyle(1001);
    tleg->SetLineStyle(0);
    tleg->SetFillColor(0);
    tleg->SetBorderSize(0);
    tleg->SetEntrySeparation(0);

    //    std::cout << "text align " << tleg->GetTextAlign() << std::endl; 

    if ( m_header!="" ) tleg->SetHeader(m_header.c_str());
    //    tleg->SetTextSize(dy*0.04*0.75/0.06);
    tleg->SetTextSize(0.06*dy*0.04*0.75/0.06/0.05);


    //    std::cout << "Legend: size() " << size() << std::endl;

    for ( unsigned i=0 ; i<m_obj.size() ; i++ ) { 
      //  std::cout << "Legend: entry " << m_labels[i] << std::endl;
      tleg->AddEntry( m_obj[i], m_labels[i].c_str(), m_type[i].c_str() );  
    }
    tleg->Draw();
  }


  double x() const { return m_x[0]; }
  double y() const { return m_y[1]; }
  
  size_t size() const { return m_labels.size(); }

private: 

  double m_x[2];
  double m_y[2];

  std::vector<std::string> m_labels;
  std::vector<std::string> m_type;
  std::vector<TObject*>    m_obj;

  std::string              m_header;

  TLegend* tleg;
  
};


